/*
 *+------------------------------------------------------------------------+
 *| Licensed Materials - Property of IBM                                   |
 *| (C) Copyright IBM Corp. 2003.  All Rights Reserved.                    |
 *|                                                                        |
 *| US Government Users Restricted Rights - Use, duplication or disclosure |
 *| restricted by GSA ADP Schedule Contract with IBM Corp.                 |
 *+------------------------------------------------------------------------+
 */
package com.ibm.datatools.validation;

import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;
import java.util.Map;

import org.eclipse.emf.ecore.EObject;

import com.ibm.db.models.db2.DB2Alias;
import com.ibm.db.models.sql.schema.Database;
import com.ibm.db.models.sql.schema.Schema;
import com.ibm.xtools.emf.validation.IValidationContext;
import com.ibm.xtools.emf.validation.internal.util.IConstraintWithExtractions;

/**
 * @author wliu
 *
 * TODO To change the template for this generated type comment go to
 * Window - Preferences - Java - Code Style - Code Templates
 */
public class AliasUniqueName implements IConstraintWithExtractions {
	private List m_elements;
	
	public AliasUniqueName() {
		super();
		m_elements = new ArrayList();
	}

	/* (non-Javadoc)
	 * @see com.ibm.xtools.emf.validation.internal.util.IConstraintWithExtractions#validate(com.ibm.xtools.emf.validation.IValidationContext, java.util.Map)
	 */
	public boolean validate(IValidationContext ctx, Map extractions) {
		m_elements.clear();
		EObject target = ctx.getTarget();
		if(target instanceof DB2Alias) {
			DB2Alias alias = (DB2Alias)target;
			
			if(findDuplicateNameInDatabase(alias)) {
				ctx.addResults(m_elements);
				Iterator dupNamesItor = m_elements.iterator();
				while(dupNamesItor.hasNext()) {
					ctx.skipCurrentConstraintFor((EObject)dupNamesItor.next());
				}
				extractions.put("aliasName", alias.getName());
				extractions.put("databaseName", alias.getSchema().getDatabase().getName());
				return false;
			}			
		}
		return true;
	}

	private boolean findDuplicateNameInDatabase(DB2Alias alias) {
		boolean hasDup = false;
		Database db = alias.getSchema().getDatabase();// alias names are unique within a database
		Iterator schemaItor = db.getSchemas().iterator();
		while(schemaItor.hasNext()) {
			Schema schema = (Schema)schemaItor.next();
			if(findDuplicateNameInSchema(alias, schema)) {
				hasDup = true;
			}
		}
		return hasDup;
	}
	
	private boolean findDuplicateNameInSchema(DB2Alias alias, Schema schema) {
		boolean hasDup = false;
		Iterator itor = schema.getTables().iterator();
		String aliasName = alias.getName();
		while(itor.hasNext()) {
			Object obj = itor.next();
			if(obj instanceof DB2Alias) {
				DB2Alias anAlias = (DB2Alias)obj;
				String anAliasName = anAlias.getName();
				if(anAliasName.equals(aliasName) && (!alias.equals(anAlias))) {
					m_elements.add(alias);
					hasDup = true;
				}
			}
		}
		return hasDup;
	}
}
